//************************************************************************
// T6963.C
// LowLevel Routinen fr LCD-Displays mit T6963
// Getestet mit AVR/ATMega bis 8MHz.
//
// Der Nullpunkt des Displays liegt oben,links. Fr unten rechts
// mssen dann zustzliche Umrechnungen stattfinden.
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 01.02.2006 t6963_busy nur als LOCALE Variable verwenden !
//            Der Wert soll nach verlassen einer Funktion gar
//            nicht erhalten bleiben !
//
// !!!!!!!!!!!!!!!!!! Achtung !!!!!!!!!!!!!!!!!!!!!
// PORTF wurde auf PORTC gendert um WR und RD ber schnelle
// sbi(), cbi() Funktionen steuern zu knnen !         
//
// 27.01.2006
// Hoch optimierte Version. Viele Funktionen durch Macro's ersetzt.
// Der Programmcode wird dadurch grer, aber auch schneller.
//
// Compiler AVR-GCC 3.4.3
//************************************************************************

#include <avr/io.h>

#include "protos.h"
#include "T6963.h"

unsigned char xcursor,ycursor;
unsigned char cgram[9];  //cgram[8] ist die CG-RAM Adresse, 0..7 die Daten

//##########################################################
//Schreibt ein Datenbyte
void WriteData(unsigned char dat)
{
 unsigned char t6963_busy;
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA(dat);
}

//##########################################################
//Schreibt ein Kommandobyte
void WriteCommand(unsigned char command)
{
 unsigned char t6963_busy;
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(command);
}

//##########################################################
//Schreibt ein Datenbyte und ein Kommandobyte
//Bisher ist das Kommandobyte immer 0xC0 fr Write+Autoincrement
//Deshalb keine Parameterbergabe fr 'command' mehr !
void WriteData1(unsigned char dat)
{
 unsigned char t6963_busy;
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA(dat);

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(AUTO_INCREMENT);
}

//##########################################################
//Schreibt zwei Datenbytes und ein Kommandobyte
void WriteData2(unsigned int dat, unsigned char command)
{
 unsigned char t6963_busy;
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)dat);
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(dat>>8));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(command);
}

//##########################################################
//Pixelarray bis zu 8 Bit auf dem Display setzen
void SetPixelArray(unsigned char xpos, unsigned char ypos, unsigned char *dat, unsigned char len)
//##########################################################
{
 unsigned int adress;
 unsigned char i;
 unsigned char t6963_busy;

//Position im Display-RAM setzen
 adress=(unsigned int)G_BASE; //Start of graphicarea

//anhand von xpos und ypos ermitteln welches Byte
//gelesen/geschrieben werden soll
// adress+= (unsigned int)BYTES_PER_ROW * (unsigned int)ypos;
// adress+= (unsigned int)xpos / (unsigned int)PIXPERBYTE;

 adress+= BYTES_PER_ROW * (unsigned int)ypos;
 adress+= xpos / PIXPERBYTE;

//########## Beginn Adresse schreiben #############################
//Set address pointer
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(adress));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(adress>>8));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(ADRESS_SET);

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(AUTO_WRITE);

//########## Ende Adresse schreiben #############################

//Note: Status check STA3 from now on !
//########## Beginn Loop Daten schreiben ########################
 for(i=0; i<len; i++)
  {
   CHECK_BUSY(CHECK_STA3);
   WRITE_T6963_DATA(*dat++);
  } //for(i=0; i<len; i++)
//########## Ende Loop Daten schreiben ########################

 CHECK_BUSY(CHECK_STA3);
 WRITE_T6963_COMMAND(AUTO_RESET);
}

//##########################################################
void DisplayOn()
//##########################################################
{
 SET_WR_DDR();
 SET_RD_DDR();
 SET_CE_DDR();
 CTRL_DATA_DDR();
 
 WR_ON();
 RD_ON();
 DATA();

 DATA_DIR_OUT(); //DatenPort auf Ausgang zum schreiben

 CE_OFF();  //you can keep CE off the whole time !
 
//Write text home address=0x0000
 WriteData2(T_BASE,0x40);

//Write graphic home address
 WriteData2(G_BASE,0x42);

//Text area column set= Setze Textspalten 
 WriteData2(BYTES_PER_ROW,0x41);

//Graphics area column set in Bytes !!!
 WriteData2(BYTES_PER_ROW,0x43);

//Set Offset Register= Setze Startadresse CGRAM
//Offset 0x02 -> 0x1400 Startadresse fr Zeichen ab 0x80 
 WriteData(CGRAM_OFFSET);
 WriteData(0);
 WriteCommand(0x22);

//set display mode = OR
 WriteCommand(0x80);

//Text and graphic display !!
 WriteCommand(0x9C);

 ClearScreen();
}

//##########################################################
// Zeichen von Windows nach T6963 konvertieren
unsigned char ConvertText(unsigned char ch)
{
 unsigned char tmp;

 tmp=ch;

 if(tmp<0x20) { tmp+=0x80; return tmp; } //CG-Chars 0..31
 if(tmp>=0x20 && tmp<=0x7F) { tmp-=0x20; return tmp; } //Hardware Font
 
//Die case-Werte sind bezogen auf den SystemFont !!!
 switch(ch)
  {
   case 196: tmp=0x6E; break; // 
   case 228: tmp=0x64; break; // 
   case 214: tmp=0x79; break; // 
   case 246: tmp=0x74; break; // 
   case 220: tmp=0x7A; break; // 
   case 252: tmp=0x61; break; // 
//   case 223: tmp=223; break; // 
   default: tmp=0xFF; break;
  }

 return tmp;
}

//##########################################################
// Kompletten TextBildschirm lschen
void ClearTextScreen()
//##########################################################
{
 int i;
 unsigned char t6963_busy;

//Write text home address
//Set address pointer = Text home
 WriteData2(T_BASE,0x24);

//ClearText RAM
 for(i=0; i<(BYTES_PER_ROW * LCD_HEIGHT/8); i++)
  {
   CHECK_BUSY(CHECK_STA0_STA1);
   WRITE_T6963_DATA(0x00);

   CHECK_BUSY(CHECK_STA0_STA1);
   WRITE_T6963_COMMAND(AUTO_INCREMENT);
  } 
}

//##########################################################
// Lsche Bereich im Grafikschirm
void ClearGScreen(unsigned int begin, unsigned int end)
//##########################################################
{
 unsigned int i;
 unsigned char t6963_busy;

 WriteData2(begin,0x24); //Setze Startadresse des zu lschenden Bereiches

 for(i=begin; i<=end; i++)
  {
   CHECK_BUSY(CHECK_STA0_STA1);
   WRITE_T6963_DATA(0x00);

   CHECK_BUSY(CHECK_STA0_STA1);
   WRITE_T6963_COMMAND(AUTO_INCREMENT);
  } 
}

//##########################################################
//Lscht den TextBildschirm, den Grafikschirm und setzt den
//Grafikcursor auf 0,0
void ClearScreen()
//##########################################################
{
 ClearGScreen(G_BASE,G_END1);
 ClearTextScreen();

//Write text home address
//Set address pointer = Text home
 LCDPos(1,1);
}

//##########################################################
// Textcursor positionieren an Position zeile,spalte
void LCDPos(unsigned char zeile, unsigned char spalte)
{
 unsigned char by;
 unsigned int tmp;
 unsigned char t6963_busy;

 if(spalte>COLS || zeile>ROWS) return; //Position auerhalb des Displays

 by=zeile-1;
 
 tmp=COLS*(unsigned int)by;
 tmp+=(spalte-1);

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(tmp));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(tmp>>8));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(ADRESS_SET);


 xcursor=spalte;
 ycursor=zeile;
}

//##########################################################
// Einen Buchstaben schreiben.
void LCDChar(unsigned char ch)
{
 unsigned char ps;
 unsigned char t6963_busy;

 ps=ch;

 ps=ConvertText(ps); //bersetzungstabelle aufrufen
 if(ps!=0xFF)
  {
   //verhindern das auerhalb des Displays geschrieben wird
   if(xcursor<=COLS && ycursor<=ROWS)
    {
     CHECK_BUSY(CHECK_STA0_STA1);
     WRITE_T6963_DATA(ps);

     CHECK_BUSY(CHECK_STA0_STA1);
     WRITE_T6963_COMMAND(AUTO_INCREMENT);

     xcursor++;  //Cursor ein Zeichen weiter
    } 
  
  }//if(ps!=0xFF)
}

//##########################################################
//selbstdefinierte Zeichen ins Character RAM schreiben
//die Zeichen mssen vorher ins Array cgram[] gelegt werden
//cgram[8] ist die Nummer unter der das Zeichen angesprochen
//werden soll minus 0x80. Dann folgen die Bytes von oben
//im Zeichen nach unten
void MakeCGRam(void)
{
 unsigned char i,by;

 by=cgram[0]*8;

 WriteData(by); //Daten schreiben

 WriteData(CGRAM_START); //Daten schreiben

 WriteCommand(0x24); //Kommando schreiben

 for(i=1; i<9; i++) WriteData1(cgram[i]);

 LCDPos(ycursor,xcursor);  //extrem wichtig ! Textcursor wiederherstellen !
}

/*
//##########################################################
// Set graphic screen base line in virtual screen
void SetGBaseLine(unsigned char line)
//##########################################################
{
 unsigned int adr;
 
 adr=G_BASE;
 adr+=BYTES_PER_ROW * line;

 WriteData2(adr,0x42); //set new startadress of screen
}
*/

//##########################################################
// Text horizontal schreiben
void LCDText(char *txt, unsigned char xpos, unsigned char ypos)
{
 char ps;

//Zeiger auf neue Textposition
 WriteData2((xpos-1+(BYTES_PER_ROW)) * (ypos-1),0x24);

 while((ps = *txt)) //Solange Zeichen grer 0x00
  {
   txt++; //Nchstes Zeichen
   if (ps== 0) break; //Raus wenn Zeichen gleich 0
   ps=ConvertText(ps); //bersetzungstabelle aufrufen
   WriteData1(ps); //Write ps, Auto Inkrement
  }
}

/*
//##########################################################
//Pixel auf dem Display setzen
//mode=0; Pixel lschen
//mode=1; Pixel setzen
void SetPixel(unsigned char xpos, unsigned char ypos, unsigned char mode)
{
 unsigned int adress;
 unsigned char tmp,pixel;

 if(xpos>=LCD_WIDTH) return; //Punkt auerhalb der Anzeige
 if(ypos>=LCD_HEIGHT) return;

//Position im Display-RAM setzen
 adress=G_BASE; //Start of graphicarea

//anhand von xpos und ypos ermitteln welches Byte
//gelesen/geschrieben werden soll
 adress+= BYTES_PER_ROW * ypos;
 adress+= xpos / PIXPERBYTE;

// if(adress>G_END) return; //Punkt auerhalb Display RAM

//Set address pointer
 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(adress));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_DATA((unsigned char)(adress>>8));

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(ADRESS_SET);

 if(mode==0) pixel=0xF0; //Kommando Pixel lschen
 else pixel=0xF8;        //Kommando Pixel setzen

//Die Reihenfolge der Pixel ist im Byte genau umgekehrt !
 tmp=(PIXPERBYTE-1)-(xpos%PIXPERBYTE);
 pixel|=tmp; //Position des Pixels im Byte setzen

 CHECK_BUSY(CHECK_STA0_STA1);
 WRITE_T6963_COMMAND(pixel);
}
*/
