//************************************************************************
// T6963.H
//
// LCD Gre und Hardwaredefinitionen
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 01.02.2006
// Compiler AVR-GCC 3.4.3
//************************************************************************
#ifndef __t6963_h
#define __t6963_h

#include "mydefs.h"

#define LCD_WIDTH	240  //Display Breite
#define LCD_HEIGHT	64   //Display Hhe

#define CGRAM_OFFSET  0x02   //CGRAM Startaddress 0x1000 for 240x64
#define CGRAM_START   0x14   //for 240x64

//#define CGRAM_OFFSET  0x03   //CGRAM Startaddress 0x1800 for 240x128
//#define CGRAM_START   0x1C   //for 240x128

// !!! only use ONE of the definitions below !!!
//#define PIXPERBYTE_6 //pin FONT of the display connected to +5V
                      //undef this if you use 8x8 builtin font of display
                
#define PIXPERBYTE_8  //pin FONT of the display connected to GROUND
                      //undef this if you use 6x8 builtin font of display

#ifdef PIXPERBYTE_6       // 6 bits per character for builtin font
 #define PIXPERBYTE    6  // than also 6 pixel per byte for graphic
 #undef PIXPERBYTE_8      // don't change PIXPERBYTE below ;)
#endif

#ifdef PIXPERBYTE_8      // 8 bits per character for builtin font
 #define PIXPERBYTE    8 // than also 8 pixel per byte for graphic
#endif

#define BYTES_PER_ROW   LCD_WIDTH / PIXPERBYTE     // how many bytes per row on screen
#define DISPLAYBYTES    BYTES_PER_ROW * LCD_HEIGHT // how many bytes for a screen

#define COLS	      LCD_WIDTH/PIXPERBYTE	//Text cols
#define ROWS	      LCD_HEIGHT/8              //Text rows

#define T_BASE  0 	//begin of textarea
#define T_END   T_BASE + ( BYTES_PER_ROW * LCD_HEIGHT / 8 ) //end of textarea

//#define G_BASE  0x0200            	// base address of first screen
#define G_BASE  0x0300            	// changed for 240x128, up to 640 chars
#define G_END  	G_BASE+DISPLAYBYTES-1   // end adress of first screen
#define G_BASE1 G_BASE+DISPLAYBYTES     // base address of 2nd screen
#define G_END1 	G_BASE1+DISPLAYBYTES-1  // end adress of 2nd screen

//WinAVR 3.4.3 setzt die Bitsetzbefehle sbi(),cbi() ins Assembler-Listing fr diese
//beiden #defines ein ! Also kein Grund Inline Assembler zu benutzen.
#define NOP() asm volatile ("nop" ::)
#define sbi(portn, bitn) ((portn)|=(1<<(bitn)))
#define cbi(portn, bitn) ((portn)&=~(1<<(bitn)))
//#define sbi(portn, bitn) asm volatile("sbi %0, %1" : : "I" (_SFR_IO_ADDR(portn)), "I" ((uint8_t)(bitn)))
//#define cbi(portn, bitn) asm volatile("cbi %0, %1" : : "I" (_SFR_IO_ADDR(portn)), "I" ((uint8_t)(bitn)))

/*
#if defined (__AVR_ATmega32__) || defined (__AVR_ATmega323__)
 //PORTC ist der Datenport

 #define DATA_DIR_IN()  DDRC=0x00; // set io-pins to inputs
 #define DATA_DIR_OUT() DDRC=0xFF; // set io-pins to outputs
 #define READ_DATA()    PINC	  // read PINC, ! NOT ! PORTC
 #define WRITE_DATA(a)  PORTC=(a); // write to data port

 #define WR_ON()  sbi(PORTA,6); //PA6
 #define WR_OFF() cbi(PORTA,6);
 #define RD_ON()  sbi(PORTA,7); //PA7
 #define RD_OFF() cbi(PORTA,7);

 #define CE_ON()  sbi(PORTD,4); //PD4
 #define CE_OFF() cbi(PORTD,4);

 #define CTRL()   sbi(PORTD,5); //PD5 => A0 => C/D = 1
 #define DATA()   cbi(PORTD,5); //       A0 => C/D = 0
#elif defined (__AVR_ATmega128__) || defined (__AVR_ATmega64__)
*/
#if defined (__AVR_ATmega128__) || defined (__AVR_ATmega64__)
 #define DATA_DIR_IN()  DDRB=0x00; // set io-pins to inputs
 #define DATA_DIR_OUT() DDRB=0xFF; // set io-pins to outputs
 #define READ_DATA()    PINB	  // read PIN, ! NOT ! PORT
 #define WRITE_DATA(a)  PORTB=(a); // write to data port

 #define WR_ON()  sbi(PORTC,6);
 #define WR_OFF() cbi(PORTC,6);
 #define SET_WR_DDR() sbi(DDRC,6);

 #define RD_ON()  sbi(PORTC,7);
 #define RD_OFF() cbi(PORTC,7);
 #define SET_RD_DDR() sbi(DDRC,7);

 #define CE_ON()  sbi(PORTD,5);
 #define CE_OFF() cbi(PORTD,5);
 #define SET_CE_DDR() sbi(DDRD,5);

 #define CTRL()   sbi(PORTD,4); //A0 => C/D = 1
 #define DATA()   cbi(PORTD,4); //A0 => C/D = 0
 #define CTRL_DATA_DDR()   sbi(DDRD,4); //A0 => C/D = 1
#else
#  error "processor type not defined in t6963.h"
#endif

//Bitmasks for BusyCheck()
#define CHECK_STA0_STA1		(unsigned char)0x03
#define CHECK_STA3		(unsigned char)0x08

//T6963 Command Set
#define AUTO_INCREMENT		0xC0
#define ADRESS_SET		0x24
#define AUTO_WRITE		0xB0
#define AUTO_READ		0xB1
#define AUTO_RESET		0xB2

//###################################################################
// Some useful macro's
//###################################################################
#ifdef FCPU4000000
#define READ_T6963()   { \
                        DATA_DIR_IN(); \
                        RD_OFF(); \
                        t6963_busy=READ_DATA(); \
                        RD_ON(); \
                       };

#define WRITE_T6963(b) { \
                        DATA_DIR_OUT();\
                        WRITE_DATA((b));\
                        WR_OFF();\
                        WR_ON();\
                       };
                       
#define CHECK_BUSY(c) { \
                       CTRL(); \
                       DATA_DIR_IN(); \
                       do{ \
                          RD_OFF(); \
                          t6963_busy=READ_DATA(); \
                          RD_ON(); \
                         }while((t6963_busy & (c)) != (c)); \
                      };
#endif //#ifdef FCPU4000000

#ifdef FCPU8000000
#define READ_T6963()   { \
                        DATA_DIR_IN(); \
                        RD_OFF(); \
                        NOP(); \
                        t6963_busy=READ_DATA(); \
                        RD_ON(); \
                       };

#define WRITE_T6963(b) { \
                        DATA_DIR_OUT();\
                        WRITE_DATA((b));\
                        WR_OFF();\
                        NOP(); \
                        WR_ON();\
                       };
                       
#define CHECK_BUSY(c) { \
                       CTRL(); \
                       DATA_DIR_IN(); \
                       do{ \
                          RD_OFF(); \
                          NOP(); \
                          t6963_busy=READ_DATA(); \
                          RD_ON(); \
                         }while((t6963_busy & (c)) != (c)); \
                      };
#endif //#ifdef FCPU8000000

#ifdef FCPU16000000
#define READ_T6963()   { \
                        DATA_DIR_IN(); \
                        RD_OFF(); \
                        NOP(); \
                        NOP(); \
                        t6963_busy=READ_DATA(); \
                        RD_ON(); \
                        NOP(); \
                       };

#define WRITE_T6963(b) { \
                        DATA_DIR_OUT();\
                        WRITE_DATA((b));\
                        WR_OFF();\
                        NOP(); \
                        NOP(); \
                        WR_ON();\
                       };
                       
//you only need two NOP() here, but it seems to be better/faster
//to wait a bit before busy check. Maybe busy is ready than
//and you come through the loop in one pass !
#define CHECK_BUSY(c) { \
                       CTRL(); \
                       DATA_DIR_IN(); \
                       do{ \
                          RD_OFF(); \
                          NOP(); \
                          NOP(); \
                          NOP(); \
                          NOP(); \
                          t6963_busy=READ_DATA(); \
                          RD_ON(); \
                         }while((t6963_busy & (c)) != (c)); \
                      };
#endif //#ifdef FCPU16000000

#define WRITE_T6963_COMMAND(d) { \
                                CTRL(); \
                                WRITE_T6963((d)); \
                               };

#define WRITE_T6963_DATA(e) { \
                                DATA(); \
                                WRITE_T6963((e)); \
                               };

//Prototypes
extern unsigned char cgram[];  //cgram[8] ist die CG-RAM Adresse, 0..7 die Daten
//extern unsigned char t6963_busy;

extern unsigned char ReadDisplay(void);
extern void WriteData(unsigned char dat);
extern void WriteCommand(unsigned char command);
extern void WriteData1(unsigned char);
extern void WriteData2(unsigned int, unsigned char);
extern void SetPixel(unsigned char xpos, unsigned char ypos, unsigned char mode);
extern unsigned char GetPixel(unsigned char xpos, unsigned char ypos);
extern void SetGBaseLine(unsigned char line);
extern void SetPosition(unsigned char xpos, unsigned char ypos);
extern void DisplayOn(void);
extern void ClearScreen(void);
extern void ClearGScreen(unsigned int begin, unsigned int end);
extern void ClearTextScreen(void);
extern void FillScreen(void);
extern unsigned char ConvertText(unsigned char ch);
extern void LCDText(char *txt, unsigned char xpos, unsigned char ypos);
extern void LCDChar(unsigned char ch);
extern void MakeCGRam(void);
extern void LCDPos(unsigned char zeile, unsigned char spalte);
extern void SetPixelArray(unsigned char xpos, unsigned char ypos, unsigned char *dat, unsigned char len);

#endif // t6963_h
