//###################################################################################
// File: ftditest.c
//
// Last change: 24.01.2006
//###################################################################################
// Compiler AVR-GCC 3.4.3
//###################################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
//###################################################################################

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "mydefs.h"
#include "ftdi245.h"
#include "t6963.h"
#include "protos.h"

unsigned int led_count;

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 unsigned char ch,col,row,i;
 unsigned char nextstate;
 unsigned char fillcount,fillend;
 unsigned char pixbuffer[256];
 unsigned char bmpheight,bmpyoffset;
 unsigned int adress;
 unsigned char xpos,ypos,pixlen;
 unsigned char t6963_busy;

 DDRB=0; 	//LCD Datenbus. Erstmal alles Eingnge
 PORTB=0x00; 	//

 DDRA=0; 	//FTDI Datenbus. Erstmal alles Eingnge
 PORTA=0x00; 	//

 DDRD=0xF2;  	// set io-pins on PortD
 PORTD=0xC2; 	//LCD-Steuersignale, FT245 Steuersignale

 DDRE=0xC0;     //The two LED

 FTDIInit();
 DisplayOn();
 LCDText("USB-T6963",1,1);
 LCDPos(1,1);
 
 nextstate=NO_STATE; //Reset State Machine

 col=0; row=0; fillcount=0; fillend=0; //only to avoid compiler warnings
 bmpheight=0; bmpyoffset=0;
 led_count=2000;
 xpos=0; ypos=0; pixlen=0;
 
 for(;;) // Anfang Programm Endlosschleife
  {
   if(led_count==1000)
    {
     GREEN_ON();
     RED_OFF();
    }
    
   if(led_count==0)
    {
     GREEN_OFF();
     RED_ON();
     led_count=2000;
    }
   led_count--;
  
   FTRead();
   ch=rxbyte;

   if(nextstate!=NO_STATE) //command and parameters already received, lets get data
    {
     switch(nextstate)
      {
       case STATE_GETBITMAP_FULLSCREEN:

            CHECK_BUSY(CHECK_STA3);
            WRITE_T6963_DATA(ch);   //write FIRST byte of the bitmap
            xpos++;

            do //outer coloumn loop
             {
                
              while(xpos<pixlen) //inner row loop
               {
                FTRead();
                CHECK_BUSY(CHECK_STA3);
                WRITE_T6963_DATA(rxbyte);
                xpos++;
               }

              if(xpos>=BYTES_PER_ROW) //one row written ?
               {
                xpos=0;
                ypos++;               //next row
               }

             }while(ypos < bmpheight); //last row of bitmap drawn ?

             CHECK_BUSY(CHECK_STA3);
             WRITE_T6963_COMMAND(AUTO_RESET); //end AUTO WRITE mode
             nextstate=NO_STATE;

       break;

       case STATE_GFXBUFFER_FILL:
            pixbuffer[fillcount]=ch;
            fillcount++;

            while(fillcount<fillend)
             {
              FTRead();
              pixbuffer[fillcount]=rxbyte;
              fillcount++;
             }
                 
            if(fillcount>=fillend)
             {
              SetPixelArray(col,row,pixbuffer,fillcount);
              nextstate=NO_STATE;
             } 
            //else keep state
       break;

       case STATE_GETBITMAP_GETROW:
            pixbuffer[fillcount]=ch; //store first byte of the row
            fillcount++;

            while(fillcount<fillend) //store all following bytes of the row
             {
              FTRead();
              pixbuffer[fillcount]=rxbyte;
              fillcount++;
             }

            if(fillcount>=fillend) //plot the row if all bytes received
             {
              SetPixelArray(col,row,pixbuffer,fillend);
              fillcount=0; //start next row
              row++;       //next row position
             }
            //else keep state

            if(row >= (bmpheight+bmpyoffset)) //last row of bitmap drawn ?
             {
              nextstate=NO_STATE;
              fillcount=0;
             }
       break;

           case STATE_GETBITMAP_X:
                col=ch; xpos=ch;
                FTRead();
                row=rxbyte; ypos=rxbyte; bmpyoffset=rxbyte;
                FTRead();
                fillend=rxbyte; pixlen=rxbyte;
                FTRead();
                bmpheight=rxbyte;
                fillcount=0;

                if(pixlen==BYTES_PER_ROW && bmpheight==LCD_HEIGHT
                   && xpos==0 && ypos==0 ) //here comes a full screen bitmap !
                 {
                  //Position 0 im Display-RAM setzen
                  adress=(unsigned int)G_BASE; //Start of graphicarea

                  //set gfx adress only ONE time !
                  CHECK_BUSY(CHECK_STA0_STA1);
                  WRITE_T6963_DATA((unsigned char)(adress));

                  CHECK_BUSY(CHECK_STA0_STA1);
                  WRITE_T6963_DATA((unsigned char)(adress>>8));

                  CHECK_BUSY(CHECK_STA0_STA1);
                  WRITE_T6963_COMMAND(ADRESS_SET);

                  CHECK_BUSY(CHECK_STA0_STA1);     //start AUTO WRITE
                  WRITE_T6963_COMMAND(AUTO_WRITE);

                  nextstate=STATE_GETBITMAP_FULLSCREEN;
                 }
                else
                 { 
                  nextstate=STATE_GETBITMAP_GETROW; //receive data on one row only
                 }

           break;
            
       default: nextstate=NO_STATE; break;
      }//switch(nextstate)

     continue;      
    }//if(nextstate!=NO_STATE)

   if( nextstate==NO_STATE && (ch<CLEAR_SCREEN || ch>LAST_COMMAND) )
    {
     LCDChar(ch); //Alles andere ist Text
     continue;
    }
       
   if( nextstate==NO_STATE && ch>=CLEAR_SCREEN && ch<LAST_COMMAND )
    {
     nextstate=STATE_COMMAND;  
    }
       
   if(nextstate==STATE_COMMAND)
    {
     switch(ch)
      {
       case SET_GFXBUFFER:  //Pixel array adress X
            FTRead(); col=rxbyte;
            FTRead(); row=rxbyte;
            FTRead(); fillend =rxbyte;   //number of bytes in buffer
            fillcount = 0; //buffer index
            nextstate=STATE_GFXBUFFER_FILL;
       break;
       case SET_BITMAP: nextstate=STATE_GETBITMAP_X; break;   //start adress X
       case SET_GFX:  //Pixel array adress X
            FTRead(); col=rxbyte;
            FTRead(); row=rxbyte;
            FTRead(); 
            SetPixelArray(col,row,&rxbyte,1);
            nextstate=NO_STATE;
       break;

       case SET_CURSOR: //Go To Text Position, Col, Row
            FTRead();
            col=rxbyte;
            FTRead();
            //row=rxbyte;
            LCDPos(rxbyte,col);
            nextstate=NO_STATE;
       break;

       case SET_CGCHAR: //Create Custom Character, 1 Byte Zeichennummer (0-7), 8 Databytes
            for(i=0; i<9; i++)
             {
              FTRead();
              cgram[i]=rxbyte;
             }
            MakeCGRam(); //CG-RAM Daten setzen
            nextstate=NO_STATE;
       break;
       case CLEAR_SCREEN: ClearScreen(); nextstate=NO_STATE; break; //Clear Display
       case CURSOR_HOME: LCDPos(1,1); nextstate=NO_STATE; break; //Go To Top Left
       case SET_CONTRAST: //Contrast, Parameter abholen
            FTRead();
//            contrast=rxbyte;
            nextstate=NO_STATE;
       break;
       case SET_LIGHT: //Backlight On, Parameter abholen
            FTRead();
//            light=rxbyte;
            nextstate=NO_STATE;
       break;
       default: nextstate=NO_STATE; break;
      }//switch ch
          
     continue;
    }//if(nextstate==STATE_COMMAND)


  } // Ende Programm Endlosschleife
}

//###################################################################################
//Nicht sonderlich genau !!
void Delay1ms(unsigned int time)
//###################################################################################
{
 unsigned int i;

 for(i=0; i<time; i++) Delay100us(10);
}

//###################################################################################
//Nicht sonderlich genau !!
//time<=255 !!
void Delay100us(unsigned char time)
//###################################################################################
{
 unsigned char k,l;

 for(k=0; k<time; k++)
  {
   for(l=0; l<40; l++)
    {
     //Ein CPU-Clock = 0.125us bei 8MHz
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
    }
  }
}

//###################################################################################
//Nicht sonderlich genau !!
//time<=255 !!
void Delay10us(unsigned char time)
//###################################################################################
{
 unsigned char k,l;

 for(k=0; k<time; k++)
  {
   for(l=0; l<4; l++)
    {
     //Ein CPU-Clock = 0.125us bei 8MHz
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
    }
  }
}
